
#include "VKBaseData.hh"

#include <TParameter.h>

using namespace std;


VKBaseData::VKBaseData() 
 : fAddRes(false)
{
    // coefficients of the energy-resolution function
    fP0 = 30.44;
    fP1 = 4.159;
}


VKBaseData::~VKBaseData() 
{
    // make sure ROOT file has been closed
    if (fOutputFile != NULL && fOutputFile->IsOpen()) fOutputFile->Close();

    // delete pointers
    delete fOutputFile;
    delete fOutputTree;
}


void VKBaseData::CreateOutput(string filename) 
{
    cout << " Creating output file: " << filename << " ..." << endl;

    // random number generator
    fRNG = new TRandom3();

    // create tree
    fOutputFile = new TFile(filename.c_str(), "RECREATE");
    fOutputTree = new TTree("Detector", "Detector data"); 
    CreateOutputBranches(fOutputTree);
    
    // write normalization
    TParameter<double>* pNorm = new TParameter<double>("NORMALIZATION", this->GetNormalization());
    fOutputTree -> GetUserInfo() -> Add(pNorm);
    cout << " Normalization: " << this->GetNormalization() << endl;
    
    // write area
    TParameter<double>* pArea = new TParameter<double>("SOURCE_AREA/cm2", this->GetArea());
    fOutputTree -> GetUserInfo() -> Add(pArea);    
    
    // loop over events and fill output tree
    for (size_t i=0; i<this->GetEntries(); i++) {

        // get event data
        const auto& ev = this -> GetEvent(i);
        _ESout = ev.ES;
        _EVout = ev.EV;
        _PSout = ev.PS;
        _PVout = ev.PV;
        _EF2out = ev.EF2;
        _ES2out = ev.ES2;
        _EV2out = ev.EV2;
        
        // sample experimental resolution
        SampleResolution(_ESout);
        SampleResolution(_EVout);
        SampleResolution(_EF2out);
        SampleResolution(_ES2out);
        SampleResolution(_EV2out);

        // fill tree
        fOutputTree -> Fill();
    }

    // write tree to file
    fOutputTree -> Write();
    
    // close output file
    fOutputFile -> Close();
    
    cout << " Done!" << endl;
}


void VKBaseData::CreateOutputBranches(TTree * t) 
{
    t->Branch("EDepSignal", &_ESout, "_ESout/F");
    t->Branch("EDepVeto"  , &_EVout,   "_EVout/F");
    t->Branch("PhotonsSignal", &_PSout, "_PSout/i");
    t->Branch("PhotonsVeto"  , &_PVout  , "_PVout/i");
    t->Branch("EDepV2Front", &_EF2out, "_EF2out/F");
    t->Branch("EDepV2Signal", &_ES2out, "_ES2out/F");
    t->Branch("EDepV2Veto"  , &_EV2out,   "_EV2out/F");
}


void VKBaseData::SampleResolution(Float_t& e)
{
    if (!fAddRes) return;
    
    double fwhm = sqrt(pow(fP0,2) + pow(fP1,2) * e);
    double sigma = fwhm / 2.35;
    
    if (e > 0) e += fRNG -> Gaus(0, sigma);

    if (e<=0) e = 0;
}
