//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
#ifndef VKBetaDetector_H
#define VKBetaDetector_H 1

#include "G4PVPlacement.hh"
#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4Sphere.hh"
#include "G4Material.hh"
#include "G4LogicalVolume.hh"
#include "G4OpticalSurface.hh"
#include "G4ThreeVector.hh"
#include "G4UnionSolid.hh"

#include "VKDetectorConstruction.hh"
#include "VKMaterials.hh"

class VKBetaDetector : public G4PVPlacement
{
  public:

    /*
     * Construct Scionix plastic scintillator v1.
     * @param pRot Orientation of the detector. If pRot==0 the detector 
     *             is oriented along the symmetry axis (z axis) looking
     *             in the direction of the source (negative z) 
     * @param tlate Position of the front size of the detector (including coating)
     * @param pMotherLogical Place detector inside this volume
     * @param pMany Boolean operations
     * @param pCopyNo Copy number
     * @param pDetConstr Pointer to main detector constructor
     */
    VKBetaDetector(G4RotationMatrix *pRot,
             const G4ThreeVector &tlate,
             G4LogicalVolume *pMother_log,
             G4bool pMany,
             G4int pCopyNo,
             const VKDetectorConstruction* pDetConstr);

    inline G4LogicalVolume* GetPhotocathodeLogVol() {return fPhotocathode_log;}
    inline G4LogicalVolume* GetSignalLogVol()       {return fSignal_log;}
    inline G4LogicalVolume* GetVetoLogVol()         {return fVeto_log;}
    
    const G4VPhysicalVolume* GetSignalPhysVol() const;
    const G4VPhysicalVolume* GetVetoPhysVol() const;
    
    inline const std::vector<G4ThreeVector>& GetSiPMPositions() const {return fSiPMPositions;}
    
  private:

    void VisAttributes();
    void SurfaceProperties();
    void SetDefaults();
    void ConstructCoatingSolid(G4double& displ); 

    const VKDetectorConstruction* fDetConstr;
    
    // Materials
    VKMaterials *fMaterials;
    G4Material* fLabVacuum;
    G4Material* fEJ200;
    G4Material* fTeflon;
    G4Material* fMylar;
    G4Material* fGlass;
    G4Material* fAluminium;    

    // Dimensions
    G4double fSignalDiameter;
    G4double fSignalLength; 
    G4double fReflectorThickness;
    G4double fVetoThickness;
    G4double fVetoLength;
    G4double fVetoEndcapLength;
    G4double fWindowThickness;
    G4double fContainerDiameter;
    G4double fContainerLength;
    G4double fSiPMDiameter;
    G4double fSiPMLength;
    
    // Geometry
    bool fCheckOverlaps;
    G4LogicalVolume* fMother_log;    
    G4Tubs* fContainer_sol;
    G4LogicalVolume* fContainer_log;

    G4Tubs* fSignal_sol;
    G4LogicalVolume* fSignal_log;

    G4UnionSolid* fVeto_sol;
    G4LogicalVolume* fVeto_log;
    
    G4Tubs* fWindow_sol;
    G4LogicalVolume* fWindow_log;

    G4UnionSolid* fCoating_sol;
    G4LogicalVolume* fCoating_log;

    G4Tubs* fSiPM_sol;
    G4LogicalVolume* fSiPM_log;

    G4Tubs* fPhotocathode_sol;
    G4LogicalVolume* fPhotocathode_log;
    
    // Surface properties
    G4double fRefl;

    // SiPM multiplicity and positions
    std::vector<G4ThreeVector> fSiPMPositions;
};

#endif
