//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
#include "VKPrimaryGeneratorMessenger.hh"
#include "VKPrimaryGeneratorAction.hh"

#include "G4UIcmdWithADoubleAndUnit.hh"
#include "G4UIcmdWith3VectorAndUnit.hh"
#include "G4UIcmdWithAnInteger.hh"
#include "G4UIcommand.hh"
#include "G4UIdirectory.hh"
#include "G4UIcmdWithABool.hh"
#include "G4UIcmdWithADouble.hh"
#include "G4UIcmdWith3Vector.hh"
#include "G4Scintillation.hh"


VKPrimaryGeneratorMessenger::VKPrimaryGeneratorMessenger(VKPrimaryGeneratorAction* pPrimGenAct)
 : fPrimGenAct(pPrimGenAct)
{
    // Setup a command directory for detector controls with guidance
    fPrimGenDir = new G4UIdirectory("/VK/particleSource/");
    fPrimGenDir -> SetGuidance("Particle source control");
       
    // Various commands for modifying the particle source
    fCRMuonAngDistCmd = new G4UIcmdWithABool("/VK/source/cosmicRayAngDist", this);
    fCRMuonAngDistCmd -> SetGuidance(
                "Enable/Disable sampling of beta-decay energy spectrum (allowed shape).");
    fCRMuonAngDistCmd -> AvailableForStates(G4State_PreInit, G4State_Idle);
    fCRMuonAngDistCmd -> SetToBeBroadcasted(false);

    fBetaSpectrumCmd = new G4UIcmdWithABool("/VK/source/betaSpectrum", this);
    fBetaSpectrumCmd -> SetGuidance(
                "Enable/Disable sampling of beta-decay energy spectrum (allowed shape).");
    fBetaSpectrumCmd -> AvailableForStates(G4State_PreInit, G4State_Idle);
    fBetaSpectrumCmd -> SetToBeBroadcasted(false);
    
    fDiskShapedSourceCmd = new G4UIcmdWithABool("/VK/source/diskShapedSource", this);
    fDiskShapedSourceCmd -> SetGuidance("Enable/Disable disk-shaped source.");
    fDiskShapedSourceCmd -> AvailableForStates(G4State_PreInit, G4State_Idle);
    fDiskShapedSourceCmd -> SetToBeBroadcasted(false);

    fBetaEndPointCmd = new G4UIcmdWithADoubleAndUnit("/VK/source/betaEndPoint", this);
    fBetaEndPointCmd -> SetGuidance("Set end-point energy of beta spectrum.");
    fBetaEndPointCmd -> SetParameterName("endPointEnergy", false);
    fBetaEndPointCmd -> SetDefaultUnit("keV");
    fBetaEndPointCmd -> AvailableForStates(G4State_PreInit, G4State_Idle);
    fBetaEndPointCmd -> SetToBeBroadcasted(false);    

    fBetaMinEnergyCmd = new G4UIcmdWithADoubleAndUnit("/VK/source/betaMinEnergy", this);
    fBetaMinEnergyCmd -> SetGuidance("Set lower bound for sampling of beta spectrum.");
    fBetaMinEnergyCmd -> SetParameterName("Emin", false);
    fBetaMinEnergyCmd -> SetDefaultUnit("keV");
    fBetaMinEnergyCmd -> AvailableForStates(G4State_PreInit, G4State_Idle);
    fBetaMinEnergyCmd -> SetToBeBroadcasted(false);    

    fBetaMaxEnergyCmd = new G4UIcmdWithADoubleAndUnit("/VK/source/betaMaxEnergy", this);
    fBetaMaxEnergyCmd -> SetGuidance("Set upper bound for sampling of beta spectrum.");
    fBetaMaxEnergyCmd -> SetParameterName("Emax", false);
    fBetaMaxEnergyCmd -> SetDefaultUnit("keV");
    fBetaMaxEnergyCmd -> AvailableForStates(G4State_PreInit, G4State_Idle);
    fBetaMaxEnergyCmd -> SetToBeBroadcasted(false);    
    
    fMinZenithCmd = new G4UIcmdWithADoubleAndUnit("/VK/source/cosmicRayMinZenithAngle", this);
    fMinZenithCmd -> SetGuidance("Set lower bound for sampling of zenith-angle distribution.");
    fMinZenithCmd -> SetParameterName("minZenith", false);
    fMinZenithCmd -> SetDefaultUnit("deg");
    fMinZenithCmd -> AvailableForStates(G4State_PreInit, G4State_Idle);
    fMinZenithCmd -> SetToBeBroadcasted(false);    

    fMaxZenithCmd = new G4UIcmdWithADoubleAndUnit("/VK/source/cosmicRayMaxZenithAngle", this);
    fMaxZenithCmd -> SetGuidance("Set upper bound for sampling of zenith-angle distribution.");
    fMaxZenithCmd -> SetParameterName("maxZenith", false);
    fMaxZenithCmd -> SetDefaultUnit("deg");
    fMaxZenithCmd -> AvailableForStates(G4State_PreInit, G4State_Idle);
    fMaxZenithCmd -> SetToBeBroadcasted(false);    

    fInnerDiameterCmd = new G4UIcmdWithADoubleAndUnit("/VK/source/innerDiameter", this);
    fInnerDiameterCmd -> SetGuidance("Set inner diameter of disk-shaped source.");
    fInnerDiameterCmd -> SetParameterName("innerDiameter", false);
    fInnerDiameterCmd -> SetDefaultUnit("mm");
    fInnerDiameterCmd -> AvailableForStates(G4State_PreInit, G4State_Idle);
    fInnerDiameterCmd -> SetToBeBroadcasted(false);    

    fOuterDiameterCmd = new G4UIcmdWithADoubleAndUnit("/VK/source/outerDiameter", this);
    fOuterDiameterCmd -> SetGuidance("Set outer diameter of disk-shaped source.");
    fOuterDiameterCmd -> SetParameterName("outerDiameter", false);
    fOuterDiameterCmd -> SetDefaultUnit("mm");
    fOuterDiameterCmd -> AvailableForStates(G4State_PreInit, G4State_Idle);
    fOuterDiameterCmd -> SetToBeBroadcasted(false);    

    fSurfaceNormalCmd = new G4UIcmdWith3Vector("/VK/source/surfaceNormal", this);
    fSurfaceNormalCmd -> SetGuidance("Set surface normal vector of disk-shaped source");
    fSurfaceNormalCmd -> AvailableForStates(G4State_PreInit, G4State_Idle);
    fSurfaceNormalCmd -> SetToBeBroadcasted(false);    

    fPowerCmd = new G4UIcmdWithADouble("/VK/source/cosmicRayPower", this);
    fPowerCmd -> SetGuidance("Set exponent of cosmic-ray angular power-law dependence");
    fPowerCmd -> SetParameterName("cosmicRayPower", false);
    fPowerCmd -> AvailableForStates(G4State_PreInit, G4State_Idle);
    fPowerCmd -> SetToBeBroadcasted(false);    
}


VKPrimaryGeneratorMessenger::~VKPrimaryGeneratorMessenger()
{
    delete fCRMuonAngDistCmd;
    delete fBetaSpectrumCmd;
    delete fBetaEndPointCmd;
    delete fBetaMinEnergyCmd;
    delete fBetaMaxEnergyCmd;
    delete fMinZenithCmd;
    delete fMaxZenithCmd;
    delete fInnerDiameterCmd;
    delete fOuterDiameterCmd;
    delete fSurfaceNormalCmd;
    delete fDiskShapedSourceCmd;
    delete fPowerCmd;
}


void VKPrimaryGeneratorMessenger::SetNewValue(G4UIcommand* command, G4String newValue)
{
    if (command == fCRMuonAngDistCmd)
    {
        fPrimGenAct -> SetCRMuonAngDistOn(fCRMuonAngDistCmd -> GetNewBoolValue(newValue));
    }
    else if (command == fBetaSpectrumCmd)
    {
        fPrimGenAct -> SetBetaSpectrumOn(fBetaSpectrumCmd -> GetNewBoolValue(newValue));
    }
    else if (command == fDiskShapedSourceCmd)
    {
        fPrimGenAct -> SetDiskShapedSourceOn(fDiskShapedSourceCmd -> GetNewBoolValue(newValue));
    }
    else if (command == fBetaEndPointCmd)
    {
        fPrimGenAct -> SetBetaEndPointEnergy(fBetaEndPointCmd -> GetNewDoubleValue(newValue));
    }
    else if (command == fBetaMinEnergyCmd)
    {
        fPrimGenAct -> SetBetaMinEnergy(fBetaMinEnergyCmd -> GetNewDoubleValue(newValue));
    }
    else if (command == fBetaMaxEnergyCmd)
    {
        fPrimGenAct -> SetBetaMaxEnergy(fBetaMaxEnergyCmd -> GetNewDoubleValue(newValue));
    }
    else if (command == fMinZenithCmd)
    {
        fPrimGenAct -> SetCRMuonMinZenithAngle(fMinZenithCmd -> GetNewDoubleValue(newValue));
    }
    else if (command == fMaxZenithCmd)
    {
        fPrimGenAct -> SetCRMuonMaxZenithAngle(fMaxZenithCmd -> GetNewDoubleValue(newValue));
    }
    else if (command == fInnerDiameterCmd)
    {
        fPrimGenAct -> SetSourceInnerDiameter(fInnerDiameterCmd -> GetNewDoubleValue(newValue));
    }
    else if (command == fOuterDiameterCmd)
    {
        fPrimGenAct -> SetSourceOuterDiameter(fOuterDiameterCmd -> GetNewDoubleValue(newValue));
    }
    else if (command == fSurfaceNormalCmd)
    {
        fPrimGenAct -> SetSourceNormal(fSurfaceNormalCmd -> GetNew3VectorValue(newValue));
    }
    else if (command == fPowerCmd)
    {
        fPrimGenAct -> SetCRMuonAngDistPower(fPowerCmd -> GetNewDoubleValue(newValue));
    }
}
